import json
import pwd

SCRIPT_VERSION = 1

# script to discover user account data

def GetUsers():
    return [u[0] for u in pwd.getpwall()]

def GenerateKeyEntry(entry):
    scheme, key, description = entry.split()
    return {
        "scheme" : scheme,
        "key" : key,
        "description" : description
    }
    
def AlignEntry(entry):
    if entry.lstrip().startswith('#'):
        return None

    # supported key types: dsa | ecdsa | ecdsa-sk | ed25519 | ed25519-sk | rsa
    supported_schemes = ['dsa','ecdsa','ed25519','rsa']
    for scheme in supported_schemes:
        idx = entry.find('ssh-' + scheme)
        if idx > -1:
            return entry[idx:]
    return None

def EnumerateAuthorizedKeysSingleUser(user):
    try:
        key_data = []
        with open('/home/{}/.ssh/authorized_keys'.format(user), 'rt') as f:
            for entry in f.readlines():
                entry = AlignEntry(entry)
                if entry != None:
                    key_data.append(GenerateKeyEntry(entry.rstrip('\n')))
        return key_data
    except:
        return None
    
def EnumerateAuthorizedKeys(users):
    user_data = []
    for user in users:
        key_data = EnumerateAuthorizedKeysSingleUser(user)
        if key_data:
            user_data.append(GenerateUserEntry(user, key_data))
    return user_data

def GenerateUserEntry(user, key_data):
    return {
        "userName" : user,
        "keyData" : key_data
    }

def ConstructOutput(key_data):
    return {
        'scriptVersion' : SCRIPT_VERSION,
        'userData' : key_data
    }

user_data = EnumerateAuthorizedKeys(GetUsers())
output = ConstructOutput(user_data)
print(json.dumps(output), end='')
